# Group Authentication
These routes deal with authentication.

## Register by Email [POST /v1/auth/register/email]
Register a new user with an email and password. Returns an access token. Additionally, and email will be sent to the user asking them to verify their email address.

+ Request (application/json)
    + Attributes (object)
        - Include FirstName
        - Include LastName
        - Include Email
        - Include EmailType
        - Include Password
        - Include PasswordConfirmation

+ Response 200 (application/json)
    + Attributes (object)
        - data (object)
            - Include AccessToken
            - Include TokenExpires

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - first_name (array[ValidationMessage])
                - last_name (array[ValidationMessage])
                - email (array[ValidationMessage])
                - email_type (array[ValidationMessage])
                - password (array[ValidationMessage])

## Login by Email [POST /v1/auth/login/email]
Login with the current user.

+ Request (application/json)
    + Attributes (object)
        - Include Email
        - Include Password
        - email: `george.washington@example.com` (string, required) - User Email address.
        - password: `5VphcVNdGhJi` (string, required) - User Password. Between 8 and 64 characters.

+ Response 200 (application/json)
    + Attributes (object)
        - data (object)
            - Include AccessToken
            - Include TokenExpires

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `AUTHENTICATION_FAILED` (string) - The code indicating authentication failure
            - error: `Invalid Credentials` (string) - The error message indicating invalid credentials
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - password (array[ValidationMessage])
                - email (array[ValidationMessage])

## Me [GET /v1/auth/me]
Get the currently logged in user's information.

+ Response 200 (application/json)
    + Attributes (User)

## Change Password [PUT /v1/auth/password/change]
Change the user's password

+ Request (application/json)
    + Attributes (object)
        - existing_password: `oldpassword` (string, required) - The user's old password
        - new_password: `newpassword` (string, required) - The user's new password
        - new_password_confirmation: `newpassword` (string, required) - The user's new password confirmation. This must match 'new_password'.

+ Response 200 (application/json)

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_EXISTING_PASSWORD` (string) - The existing password is invalid
            - error: `Invalid existing password` (string) - Error for existing password invalidity
            - more (object)

## Forgot Password [POST /v1/auth/password/forgot]
Send an email or text message to user. The email will contain a uri to deep link the user into the application where they can then enter a new password. The deep link will embed a code and a verification type of password. This code can then be used when making the call to [Reset Password](#authentication-reset-password). Depending on the verification method, the contact parameter in the link will either be the email address or cell phone number the notification was sent to.

If the verification method of `email` is used:
- The link will be in the following format: `scale-tickets.app://scale-tickets.app/password_reset?code=12345&contact=george.washington@gmail.com`. 

If the verification method of `cell` is used:
- The link will be in the following format: `scale-tickets.app://scale-tickets.app/password_reset?code=12345&contact=7015555678`. 

+ Request (application/json)
    + Attributes (object)
        - contact_method: `email` (enum[string]) - The verification method used to send the password reset notification
            - `email`
            - `cell`
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number to send the notification to

+ Response 200 (application/json)

## Reset Password [PUT /v1/auth/password/reset]
This endpoint is to be used after calling the [Forgot Password](#authentication-forgot-password) endpoint. Upon the user successfully opening either an email message or a text message which routes them to the deep link similar to the one listed in that section, that code can be used to store a new password for the user by replacing the existing one.

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number that the verification notification was sent to
        - code: `12345` (string, required) - The password reset code that was sent to the user via their chosen notification method
        - Include Password
        - Include PasswordConfirmation

+ Response 200 (application/json)

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_CODE` (string) - The code is invalid
            - error: `Invalid CODE` (string) - Error for invalid code
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - code (array[ValidationMessage])
                - contact (array[ValidationMessage])
                - password (array[ValidationMessage])
                - password_confirmation (array[ValidationMessage])

## Signup [POST /v1/auth/signup]
Sign up for the application using the user's email address or cell phone number. The endpoint will respond with an access token and its associated expiration date. 


If the user selects contact method `email`:
- A verification email will be sent to the email address given so the user can authenticate their email address. The link will be in the following format: `scale-tickets.app://scale-tickets.app/email_verification?code=12345&contact=george.washington@gmail.com`. 

If the user selects contact method `cell`:
- A verification text message will be sent to the number given so the user can authenticate their cell phone number. The link will be in the following format: `scale-tickets.app://scale-tickets.app/cell_verification?code=12345&contact=7015551879`.

This code and contact provided in the link can then be used when making the call to [Verify Code](#authentication-verify-code)

+ Request (application/json)
    + Attributes (object)
        - contact_method: `email` (enum[string]) - The contact method used to send the notification
            - `email`
            - `cell`
        - contact: `george.washington@gmail.com` (string) - The user's email address or cell phone number to send the notification to

+ Response 200 (application/json)
    + Attributes (object)
        - data (object)
            - Include AccessToken
            - Include TokenExpires

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - contact_method (array[ValidationMessage])
                - contact (array[ValidationMessage])

## Verify Code [POST /v1/auth/verify]
This endpoint is to be used after calling the [Signup](#authentication-signup) endpoint. Upon the user successfully opening the email message or text message which routes them to the app via a deep link as described in that section, the code and contact can be used to authenticate the user to the provided email address or cell phone.

+ Request (application/json)
    + Attributes (object)
        - contact: `george.washington@gmail.com` (string, required) - The user's email address or cell phone number that the verification notification was sent to
        - code: `12345` (string, required) - The password reset code that was sent to the user via their chosen notification method

+ Response 200 (application/json)

+ Response 401 (application/json)
    + Attributes (object)
        - data (object)
            - code: `INVALID_CODE` (string) - The code is invalid
            - error: `Invalid CODE` (string) - Error for invalid code
            - more (object)

+ Response 422 (application/json)
    + Attributes (object)
        - error (object)
            - code: `VALIDATION_ERROR` (string)
            - message: `You have validation errors in your submission` (string)
            - validation_messages (object)
                - code (array[ValidationMessage])
                - contact (array[ValidationMessage])

## Data Structures

### Id (object)
- id: `1` (number) - The id of the entity

### Timestamps (object)
- created_at: `2016-05-03T10:41:31.257Z` (string) - The created date in ISO8601 format
- updated_at: `2016-05-03T10:41:31.257Z` (string) - The updated date in ISO8601 format

### ValidationMessage (object)
- message: `The value in this field is not valid.` (string) - The validation error message

### AccessToken (object)
- access_token: `48a4034027374be582b936f7d58043af` (string) - The access token to be used to authenticate future requests

### TokenExpires (object)
- expires: `2016-05-03T11:15:54.950Z` (string, required) - The datetime when the access token will expire

### Email (object)
- email: `george.washington@example.com` (string, required) - User Email address.

### EmailType (object)
- email_type: `personal` (enum[string], required) - The User Email type
    - `work`
    - `personal`

### Phone (object)
- phone: `555-555-5555` (string, required) - User Email address.

### PhoneType (object)
- phone_type: `personal` (enum[string], required) - The User Phone type
    - `work`
    - `cell`
    - `home`

### FirstName (object)
- first_name: `George` (string, required) - User First Name.

### LastName (object)
- last_name: `Washington` (string, required) - User Last Name.

### Password (object)
- password: `5VphcVNdGhJi` (string, required) - User Password. Between 8 and 64 characters.

### PasswordConfirmation (object)
- password_confirmation: `5VphcVNdGhJi` (string, required) - User Password test.  Must equal property 'password'.

### Primary (object)
- primary: `true` (boolean) - This is user's primary record of its type

### Verified (object)
- verified: `false` (boolean) - The user has verified this record

### UserEmail (object)
- Include Id
- Include Email
- Include EmailType
- Include Primary
- Include Verified
- Include Timestamps

### UserPhone (object)
- Include Id
- Include Phone
- Include PhoneType
- Include Primary
- Include Verified
- Include Timestamps


### User (object)
- Include Id
- Include FirstName
- Include LastName
- last_login: `2016-05-03T12:21:25.858Z` (string) - The date of the user's last successful login
- phones (object)
    - data (array[UserPhone])
- emails (object)
    - data (array[UserEmail])
- Include Timestamps
